from django.shortcuts import render, redirect, get_object_or_404
from django.contrib import messages
from django.contrib.auth.decorators import login_required, user_passes_test
from django.http import HttpResponse, HttpResponseForbidden
from django.core.paginator import Paginator
from .models import Leave
from .forms import LeaveRequestForm, AdminLeaveCreationForm
from account.models import Account
from django.db.models import Q
import csv
from datetime import datetime


def admin_check(user):
    return user.is_admin_user or user.is_super_user


@login_required
def request_leave(request):
    if request.method == "POST":
        form = LeaveRequestForm(request.POST)
        if form.is_valid():
            leave = form.save(commit=False)
            leave.user = request.user
            leave.save()
            messages.success(request, 'Leave request submitted successfully.')
            return redirect('sales:leave_history')
    else:
        form = LeaveRequestForm()
    return render(request, 'sales/leave/request_leave.html', {'form': form})


@login_required
def leave_history(request):
    leaves = Leave.objects.leaves_for_user(request.user)

    # Apply filters
    status = request.GET.get('status')
    if status:
        leaves = leaves.filter(status__icontains=status)

    from_date = request.GET.get('from_date')
    to_date = request.GET.get('to_date')
    if from_date and to_date:
        leaves = leaves.filter(startdate__range=[from_date, to_date])

    # Pagination
    paginator = Paginator(leaves, 10)
    page_number = request.GET.get('page')
    page_obj = paginator.get_page(page_number)

    data = {
        "page_obj": page_obj,
    }

    if 'export' in request.GET and request.GET['export'] == 'true':
        response = HttpResponse(content_type='text/csv')
        response['Content-Disposition'] = 'attachment; filename="leave_history.csv"'

        writer = csv.writer(response)
        writer.writerow(['Start Date', 'End Date', 'Leave Type', 'Reason', 'Status', 'Created At'])

        for leave in leaves:
            writer.writerow([
                leave.startdate,
                leave.enddate,
                leave.leavetype,
                leave.reason,
                leave.status,
                leave.created,
            ])

        return response

    return render(request, 'sales/leave/request_leave.html', data)


@login_required
@user_passes_test(admin_check)
def admin_leave_dashboard(request):
    if request.user.is_admin_user or request.user.is_super_user:
        leaves = Leave.objects.all().order_by('-created')

        # Apply filters
        status = request.GET.get('status')
        if status:
            leaves = leaves.filter(status__icontains=status)

        user_email = request.GET.get('user_email')
        if user_email:
            leaves = leaves.filter(user__email__icontains=user_email)

        from_date = request.GET.get('from_date')
        to_date = request.GET.get('to_date')
        if from_date and to_date:
            leaves = leaves.filter(startdate__range=[from_date, to_date])

        # Check if any leave has completed and update its status
        today = datetime.now().date()
        for leave in leaves:
            if leave.enddate <= today and leave.status != Leave.COMPLETED:
                leave.status = Leave.COMPLETED
                leave.save()

        # Pagination
        paginator = Paginator(leaves, 10)
        page_number = request.GET.get('page')
        page_obj = paginator.get_page(page_number)

        # Get all accounts for the user dropdown
        accounts = Account.objects.all()

        data = {
            "page_obj": page_obj,
            "accounts": accounts
        }

        # Export to CSV functionality
        if 'export' in request.GET and request.GET['export'] == 'true':
            response = HttpResponse(content_type='text/csv')
            response['Content-Disposition'] = 'attachment; filename="all_leaves.csv"'

            writer = csv.writer(response)
            writer.writerow(['User', 'Start Date', 'End Date', 'Leave Type', 'Reason', 'Status', 'Created At'])

            for leave in leaves:
                writer.writerow([
                    leave.user.email,
                    leave.startdate,
                    leave.enddate,
                    leave.leavetype,
                    leave.reason,
                    leave.status,
                    leave.created,
                ])

            return response

        return render(request, 'backoffice/leave/admin_leave.html', data)
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")

@login_required
@user_passes_test(admin_check)
def approve_leave(request, leave_id):
    if request.user.is_admin_user or request.user.is_super_user:
        leave = get_object_or_404(Leave, id=leave_id)
        leave.approve_leave
        messages.success(request, 'Leave approved successfully.')
        return redirect('backoffice:admin_leave_dashboard')
    else:
        return HttpResponseForbidden("You are not authorized to perform this action.")


@login_required
@user_passes_test(admin_check)
def reject_leave(request, leave_id):
    if request.user.is_admin_user or request.user.is_super_user:
        leave = get_object_or_404(Leave, id=leave_id)
        leave.reject_leave
        messages.success(request, 'Leave rejected successfully.')
        return redirect('backoffice:admin_leave_dashboard')
    else:
        return HttpResponseForbidden("You are not authorized to perform this action.")


@login_required
@user_passes_test(admin_check)
def cancel_leave(request, leave_id):
    if request.user.is_admin_user or request.user.is_super_user:
        leave = get_object_or_404(Leave, id=leave_id)
        leave.leaves_cancel
        messages.success(request, 'Leave cancelled successfully.')
        return redirect('backoffice:admin_leave_dashboard')
    else:
        return HttpResponseForbidden("You are not authorized to perform this action.")


@login_required
@user_passes_test(admin_check)
def create_leave_for_user(request):
    if request.user.is_admin_user or request.user.is_super_user:
        if request.method == "POST":
            form = AdminLeaveCreationForm(request.POST)
            if form.is_valid():
                form.save()
                messages.success(request, 'Leave created for user successfully.')
                return redirect('backoffice:admin_leave_dashboard')
        else:
            form = AdminLeaveCreationForm()
        return render(request, 'backoffice/leave/admin_leave.html', {'form': form})
    else:
        return HttpResponseForbidden("You are not authorized to perform this action.")


@login_required
@user_passes_test(admin_check)
def leave_statistics(request):
    if request.user.is_admin_user or request.user.is_super_user:
        current_year = datetime.now().year
        yearly_leaves = Leave.objects.current_year_leaves()
        pending_leaves = Leave.objects.all_pending_leaves()
        approved_leaves = Leave.objects.all_approved_leaves()
        rejected_leaves = Leave.objects.all_rejected_leaves()
        cancelled_leaves = Leave.objects.all_cancel_leaves()

        context = {
            'current_year': current_year,
            'yearly_leaves': yearly_leaves,
            'pending_leaves': pending_leaves,
            'approved_leaves': approved_leaves,
            'rejected_leaves': rejected_leaves,
            'cancelled_leaves': cancelled_leaves,
        }

        # return render(request, 'backoffice/leave/leave_statistics.html', context)
    else:
        return HttpResponseForbidden("You are not authorized to view this page.")
